@@ -49,7 +49,7 @@ void cupboard_door_clicked(TangleAction *action, GObject *source, const gchar *t
 /*
 It suppose filenames are formatted in this way (will seg fault if not).
 folder/+theme+[123]+[ae]+date.
-example:+fantasy+2+e+2010.03.11_00.03.52.txt
+example:+fantasy+2+e+2010.03.11_00.03.52
 theme_name = city / animal / fantasy          (this is checked, affects icon graphics)
 variation = 1 / 2 / 3                         (not critical, user will get empty set of samplebuttons, if use non-valid variation)
 mode (easy/advanced) = e /a                   (if unclear, interpret advanced, because there can be two tracks)
