@@ -166,15 +166,6 @@ static void on_sequencer_stopped_playback(JammoSequencer* sequencer_old, gpointe
 	g_free(filename);
 }
 
-static void say_now_you_hear_own_singing(JammoMentor* mentor, const gchar*speech, gboolean interrupted, gpointer sequencer){
-	if (interrupted){
-		start_sequencer(NULL, NULL, interrupted, sequencer);
-		return;
-	}
-
-	jammo_mentor_speak_with_callback(jammo_mentor_get_default(), "singinggame_listen.spx", start_sequencer, sequencer);
-}
-
 
 void on_sequencer_stopped_recording(JammoSequencer* sequencer_old, gpointer user_data) {
 	ClutterActor* view;
@@ -247,7 +238,7 @@ void on_sequencer_stopped_recording(JammoSequencer* sequencer_old, gpointer user
 
 	chum_set_state_of_chum(SCREEN_PLAY_RECORDED);
 
-	jammo_mentor_speak_with_callback(jammo_mentor_get_default(), "welldone.spx", say_now_you_hear_own_singing, sequencer);
+	jammo_mentor_speak_with_callback(jammo_mentor_get_default(), "welldone.spx singinggame_listen.spx", start_sequencer, sequencer);
 }
 
 
