@@ -9,6 +9,7 @@
 #include "../jammo-cursor.h"
 #include "../jammo-mentor.h"
 #include "../jammo.h"
+#include "../save_helper.h"
 
 #include "chum.h" //chum_is_easy_game
 #include "theme.h"
@@ -68,13 +69,19 @@ static void animate_icon_to_cupboard(JammoMentor* mentor, const gchar* speech, g
 
 static gboolean save_image_pressed(TangleButton *tanglebutton, gpointer data){
 	jammo_theme* theme = (jammo_theme*)data;
-	const gchar* filename = clutter_actor_get_name(CLUTTER_ACTOR(tanglebutton));
-
 	clutter_actor_hide(jammo_get_actor_by_id("saving_dialog_cancel")); //No canceling after image is selected.
 
+	const gchar* name_of_icon = clutter_actor_get_name(CLUTTER_ACTOR(tanglebutton));
+	printf("Selected '%s'\n",name_of_icon);
 
-	printf("Selected '%s'\n",filename);
-	composition_game_file_helper_save_composition((gchar*)filename, theme->bottom_track_view, theme->upper_track_view, theme->name, theme->variation);
+	gchar* filename_for_saving = g_strdup_printf("+%s+%d+%s-",theme->name,theme->variation,chum_is_easy_game()?"e":"a");
+	//e.g.: +fantasy+2+e-
+
+	gchar* additional_metadata = g_strdup_printf("\"icon\" : \"%s\",\n\"theme\" : \"%s\",\n\"variation\" : %d,\n",name_of_icon,theme->name,theme->variation);
+
+	save_composition(filename_for_saving, jammo_get_actor_by_id("composition_game_container_for_tracks"),additional_metadata);
+	g_free(filename_for_saving);
+	g_free(additional_metadata);
 	jammo_mentor_speak_with_callback(jammo_mentor_get_default(),"save_confirmed.spx",animate_icon_to_cupboard,tanglebutton);
 	return FALSE;
 }
