@@ -444,7 +444,7 @@ static gboolean first_time = TRUE;
 /* JSON callback when sequencer-view is showed*/
 void start_sequencer_full_sequencer (TangleAction *action, GObject *source, const gchar *trigger, TangleProperties *properties){
 	if (first_time)
-		start_sequencer("seq.json"); //This is default
+		start_sequencer("/opt/jammo/seq.json"); //This is default
 	first_time=FALSE;
 }
 
