@@ -36,15 +36,20 @@ gboolean sequencer_lock_pressed (TangleButton* tanglebutton, gpointer data){
 	else if (strncmp(name,"effec",5)==0)
 		scroller = jammo_get_actor_by_id("fullsequencer-loop-effect-wheel-scroller");
 
-	else
+	else {
 		printf("can't detect which lock pressed\n");
+		return FALSE;
+	}
+
+	ClutterAction* scroll_action = tangle_actor_get_action_by_type(scroller,TANGLE_TYPE_SCROLL_ACTION);
+	gfloat threshold_x, threshold_y;
 
 	if (scroller)  {
-		gfloat y = tangle_scrolling_actor_get_scrolling_threshold_y(TANGLE_SCROLLING_ACTOR(scroller));
-		if (y==10000.0)
-			tangle_scrolling_actor_set_scrolling_threshold_y(TANGLE_SCROLLING_ACTOR(scroller),30.0);//TODO: use original default value
+		tangle_scroll_action_get_thresholds(TANGLE_SCROLL_ACTION(scroll_action), &threshold_x,&threshold_y);
+		if (threshold_y==10000.0)
+			tangle_scroll_action_set_thresholds(TANGLE_SCROLL_ACTION(scroll_action), threshold_x,30.0);//TODO: use original default value
 		else
-			tangle_scrolling_actor_set_scrolling_threshold_y(TANGLE_SCROLLING_ACTOR(scroller),10000.0);
+			tangle_scroll_action_set_thresholds(TANGLE_SCROLL_ACTION(scroll_action), threshold_x,10000.0);
 	}
 	return TRUE;
 }
@@ -54,8 +59,10 @@ gboolean sequencer_lock_pressed (TangleButton* tanglebutton, gpointer data){
 ScrollingActor doesn't have clampping. This will set it.
 */
 static void fine_tune_wheel (ClutterActor* sample_looper) {
-	ClutterActor* scrolling_actor = clutter_actor_get_parent(sample_looper);
-	g_signal_connect_swapped(scrolling_actor, "clamp-scrolling-offset-y", G_CALLBACK(tangle_widget_clamp_child_boundaries), sample_looper);
+	ClutterAction* action;
+	
+	action = tangle_actor_get_action_by_type(sample_looper, TANGLE_TYPE_SCROLL_ACTION);
+	g_signal_connect_swapped(action, "clamp-offset-y", G_CALLBACK(tangle_widget_clamp_child_boundaries), sample_looper);
 }
 
 
@@ -143,21 +150,23 @@ If loop is already rolling, this will abort it and will make own.
 Loop will still snap to grid always.
 */
 static void rotate_loop_for_amount(ClutterActor* wheel, gint amount) {
-	ClutterActor* scroller = clutter_actor_get_parent(wheel);
-	if (TANGLE_IS_SCROLLING_ACTOR(scroller)){
-		//Check if lock is pressed
-		gfloat threshold_y = tangle_scrolling_actor_get_scrolling_threshold_y(TANGLE_SCROLLING_ACTOR(scroller));
-		if (threshold_y==10000.0) //means rolling is disabled for this wheel 
-			return;
-		gfloat offset = tangle_scrolling_actor_get_scrolling_offset_y(TANGLE_SCROLLING_ACTOR(scroller));
-		int current_slot = (int)(offset)/92; //round to lower slot, if rolling already
-		gfloat target = (current_slot+amount) * 92.0;
-		//printf("current_slot = %d, target pixel='%f'\n",current_slot,target);
-		clutter_actor_animate(CLUTTER_ACTOR(scroller), CLUTTER_EASE_IN_OUT_QUAD, 550, "scrolling-offset-y", target, NULL);
-	}
-	else {
-		printf("scroller was not parent of wheel\n");
-	}
+	ClutterAction* scroll_action = tangle_actor_get_action_by_type(wheel,TANGLE_TYPE_SCROLL_ACTION);
+
+	//Check if lock is pressed
+	gfloat not_used,threshold_y;
+	tangle_scroll_action_get_thresholds(TANGLE_SCROLL_ACTION(scroll_action), &not_used,&threshold_y);
+
+	//printf("threshold_y '%f'\n",threshold_y);
+	if (threshold_y==10000.0) //means rolling is disabled for this wheel
+		return;
+	gfloat offset;
+	tangle_scroll_action_get_offsets(TANGLE_SCROLL_ACTION(scroll_action), &not_used,&offset);
+	//printf("offset '%f'\n",offset);
+
+	int current_slot = (int)(offset)/92; //round to lower slot, if rolling already
+	gfloat target = (current_slot+amount) * 92.0;
+	//printf("current_slot = %d, target pixel='%f'\n",current_slot,target);
+	tangle_object_animate(G_OBJECT(scroll_action),  CLUTTER_EASE_IN_OUT_QUAD, 550, "offset-y", target, NULL);
 }
 
 
@@ -193,7 +202,7 @@ void fullsequencer_loop_crazy_clicked (TangleAction *action, GObject *source, co
 //////////////////////////////////////
 
 //Bottom of the loop-view there are dropping-area, and when samplebutton is dragged here, view is changed.
-gboolean sequencer_handle_dragging (TangleDroppableActor *droppable_actor, TangleDragging *dragging){
+gboolean sequencer_drag_begin(TangleDropAction* drop_action, TangleDragAction* drag_action){
 	printf("jammo_sample_button dragged from wheel to bottom of the screen\n");
 	sequencer_change_to_sequencer_view(NULL,NULL);
 	
@@ -204,10 +213,10 @@ gboolean sequencer_handle_dragging (TangleDroppableActor *droppable_actor, Tangl
 void sequencer_loop_tune_wheels() {
 
 	//Static
-	rhytmical_sample_looper  =  jammo_get_actor_by_id("fullsequencer-loop-rhytmical-samples");
-	melodical_sample_looper  =  jammo_get_actor_by_id("fullsequencer-loop-melodical-samples");
-	harmonical_sample_looper =  jammo_get_actor_by_id("fullsequencer-loop-harmonical-samples");
-	effect_sample_looper     =  jammo_get_actor_by_id("fullsequencer-loop-effect-samples");
+	rhytmical_sample_looper  =  jammo_get_actor_by_id("fullsequencer-loop-rhytmical-wheel-scroller");
+	melodical_sample_looper  =  jammo_get_actor_by_id("fullsequencer-loop-melodical-wheel-scroller");
+	harmonical_sample_looper =  jammo_get_actor_by_id("fullsequencer-loop-harmonical-wheel-scroller");
+	effect_sample_looper     =  jammo_get_actor_by_id("fullsequencer-loop-effect-wheel-scroller");
 
 	fine_tune_wheel(rhytmical_sample_looper);
 	fine_tune_wheel(melodical_sample_looper);
