@@ -12,7 +12,7 @@
 #include "jammo-note-button.h"
 #include "../meam/jammo-midi.h"
 
-G_DEFINE_TYPE(JammoMiditrackView, jammo_miditrack_view, TANGLE_TYPE_DROPPABLE_ACTOR);
+G_DEFINE_TYPE(JammoMiditrackView, jammo_miditrack_view, TANGLE_TYPE_WIDGET);
 
 enum {
 	PROP_0,
@@ -126,7 +126,7 @@ Returns newly created jammo-note-button
 */
 ClutterActor* jammo_miditrack_view_add_note(JammoMiditrackView* miditrack_view, JammoMidiEvent* start, JammoMidiEvent* stop ) {
 	TangleWidget* widget;
-	widget = TANGLE_WIDGET(tangle_wrapper_actor_get_wrapped(TANGLE_WRAPPER_ACTOR(miditrack_view)));
+	widget = TANGLE_WIDGET(miditrack_view);
 
 	if (start->note != stop->note) {
 		return NULL;
@@ -182,15 +182,15 @@ gboolean jammo_miditrack_view_zoom_to_fit(JammoMiditrackView* miditrack_view, gf
 	g_object_set(miditrack_view,"slot-height", new_height,"slot-width", new_width,NULL);
 
 	//New values for each existing note
-	TangleWidget* widget = TANGLE_WIDGET(tangle_wrapper_actor_get_wrapped(TANGLE_WRAPPER_ACTOR(miditrack_view)));
+	TangleWidget* widget = TANGLE_WIDGET(miditrack_view);
 	GList* children;
 	ClutterActor* actor;
 	for (children = tangle_widget_get_children_readonly(widget); children; children = children->next) {
 		actor = CLUTTER_ACTOR(children->data);
-		clutter_actor_set_width(actor,new_width);
-		clutter_actor_set_x(actor,ratio_x*clutter_actor_get_x(actor));
-		clutter_actor_set_height(actor,new_height);
-		clutter_actor_set_y(actor,ratio_y*clutter_actor_get_y(actor));
+		clutter_actor_set_width(actor,ratio_x*clutter_actor_get_width(actor)); //old value * ratio_x
+		clutter_actor_set_x(actor,ratio_x*clutter_actor_get_x(actor));  //old value * ratio_x
+		clutter_actor_set_height(actor,new_height); //Height of note is always same than height of slot.
+		clutter_actor_set_y(actor,ratio_y*clutter_actor_get_y(actor));  //old value * ratio_y
 		}
 	return FALSE;
 }
@@ -219,7 +219,7 @@ void jammo_miditrack_view_set_editing_enabled(JammoMiditrackView* miditrack_view
 		miditrack_view->priv->editing_enabled = editing_enabled;
 		
 		tangle_droppable_actor_set_dropping_enabled(TANGLE_DROPPABLE_ACTOR(miditrack_view), editing_enabled);
-		for (children = tangle_widget_get_children_readonly(TANGLE_WIDGET(tangle_wrapper_actor_get_wrapped(TANGLE_WRAPPER_ACTOR(miditrack_view)))); children; children = children->next) {
+		for (children = tangle_widget_get_children_readonly(TANGLE_WIDGET(miditrack_view)); children; children = children->next) {
 			tangle_draggable_actor_set_dragging_enabled(TANGLE_DRAGGABLE_ACTOR(children->data), editing_enabled);
 		}
 		
@@ -227,40 +227,48 @@ void jammo_miditrack_view_set_editing_enabled(JammoMiditrackView* miditrack_view
 	}
 }
 
-static gboolean jammo_miditrack_view_is_dragging_droppable(TangleDroppableActor* droppable_actor, TangleDragging* dragging) {
+static gboolean on_drag_begin_or_motion(TangleDropAction* drop_action, TangleDragAction* drag_action, gpointer user_data) {
 	gboolean droppable = FALSE;
 	JammoMiditrackView* miditrack_view;
+	ClutterActor* meta_actor;
+	ClutterActor* drag_actor;
+	gfloat x, y, w, h;
 	gfloat center_x, center_y;
 	guint slot, slot_y;
 	guint n_slots;
 	gfloat start;
 	gfloat vertical_place;
 	
-	miditrack_view = JAMMO_MIDITRACK_VIEW(droppable_actor);
+	miditrack_view = JAMMO_MIDITRACK_VIEW(user_data);
 	
-	if (JAMMO_IS_NOTE_BUTTON(dragging->draggable_actor) &&
-	    clutter_actor_transform_stage_point(CLUTTER_ACTOR(miditrack_view), dragging->center_x, dragging->center_y, &center_x, &center_y)) {
-		n_slots = (guint)(clutter_actor_get_width(CLUTTER_ACTOR(dragging->draggable_actor)) / miditrack_view->priv->real_slot_width + 0.5);
-		center_x -= (n_slots - 1) * miditrack_view->priv->real_slot_width / 2;
-		slot = center_x / miditrack_view->priv->real_slot_width;
+	meta_actor = clutter_actor_meta_get_actor(CLUTTER_ACTOR_META(drag_action));
+	drag_actor = tangle_drag_action_get_drag_actor(drag_action);;
+	if (JAMMO_IS_NOTE_BUTTON(meta_actor)) {
+		clutter_actor_get_transformed_position(drag_actor, &x, &y);
+		clutter_actor_get_size(drag_actor, &w, &h);
+		if (clutter_actor_transform_stage_point(CLUTTER_ACTOR(miditrack_view), x + w / 2, y + h / 2, &center_x, &center_y)) {
+			n_slots = (guint)(clutter_actor_get_width(drag_actor) / miditrack_view->priv->real_slot_width + 0.5);
+			center_x -= (n_slots - 1) * miditrack_view->priv->real_slot_width / 2;
+			slot = center_x / miditrack_view->priv->real_slot_width;
 
-		slot_y = center_y/miditrack_view->priv->slot_height;
+			slot_y = center_y/miditrack_view->priv->slot_height;
+
+			if (slot >= miditrack_view->priv->disabled_slots_begin &&
+			    slot + n_slots - 1 < miditrack_view->priv->n_slots - miditrack_view->priv->disabled_slots_end) {
+				droppable = TRUE;
+				start = slot * miditrack_view->priv->real_slot_width;
+				if (start != miditrack_view->priv->hilighted_start) {
+					miditrack_view->priv->hilighted_start = start;
+					miditrack_view->priv->hilighted_end = start + n_slots * miditrack_view->priv->real_slot_width;
+					clutter_actor_queue_redraw(CLUTTER_ACTOR(miditrack_view));
+				}
+				vertical_place = (gint)(center_y / miditrack_view->priv->slot_height) * miditrack_view->priv->slot_height;
+				if (vertical_place != miditrack_view->priv->hilighted_horizontal_line){
+					miditrack_view->priv->hilighted_horizontal_line = vertical_place;
+					clutter_actor_queue_redraw(CLUTTER_ACTOR(miditrack_view));
+				}
 
-		if (slot >= miditrack_view->priv->disabled_slots_begin &&
-		    slot + n_slots - 1 < miditrack_view->priv->n_slots - miditrack_view->priv->disabled_slots_end) {
-			droppable = TRUE;
-			start = slot * miditrack_view->priv->real_slot_width;
-			if (start != miditrack_view->priv->hilighted_start) {
-				miditrack_view->priv->hilighted_start = start;
-				miditrack_view->priv->hilighted_end = start + n_slots * miditrack_view->priv->real_slot_width;
-				clutter_actor_queue_redraw(CLUTTER_ACTOR(miditrack_view));
 			}
-			vertical_place = (gint)(center_y / miditrack_view->priv->slot_height) * miditrack_view->priv->slot_height;
-			if (vertical_place != miditrack_view->priv->hilighted_horizontal_line){
-				miditrack_view->priv->hilighted_horizontal_line = vertical_place;
-				clutter_actor_queue_redraw(CLUTTER_ACTOR(miditrack_view));
-			}
-
 		}
 	}
 	
@@ -269,11 +277,7 @@ static gboolean jammo_miditrack_view_is_dragging_droppable(TangleDroppableActor*
 		clutter_actor_queue_redraw(CLUTTER_ACTOR(miditrack_view));
 	}
 	    		
-	return droppable;
-}
-
-static gboolean jammo_miditrack_view_handle_dragging(TangleDroppableActor* droppable_actor, TangleDragging* dragging) {
-	return FALSE;
+	return !droppable;
 }
 
 static void jammo_miditrack_view_get_preferred_width(TangleActor* actor, gfloat for_height, gboolean interacting, gfloat* min_width_p, gfloat* natural_width_p, gfloat* max_width_p) {
@@ -310,13 +314,13 @@ static void jammo_miditrack_view_get_preferred_height(TangleActor* actor, gfloat
 	//printf("height of miditrack: '%f'. (notes %d...%d. slot_height=%f)\n",height, miditrack_view->priv->highest_note, miditrack_view->priv->lowest_note,  miditrack_view->priv->slot_height);
 }
 
-static void jammo_miditrack_view_allocate_wrapped(TangleWrapperActor* wrapper_actor, ClutterActor* wrapped, const ClutterActorBox* box, ClutterAllocationFlags flags) {
+static void jammo_miditrack_view_allocate(ClutterActor* actor, const ClutterActorBox* box, ClutterAllocationFlags flags) {
 	JammoMiditrackView* miditrack_view;
 
-	miditrack_view = JAMMO_MIDITRACK_VIEW(wrapper_actor);
+	miditrack_view = JAMMO_MIDITRACK_VIEW(actor);
+
+	CLUTTER_ACTOR_CLASS(jammo_miditrack_view_parent_class)->allocate(actor, box, flags);
 
-	TANGLE_WRAPPER_ACTOR_CLASS(jammo_miditrack_view_parent_class)->allocate_wrapped(wrapper_actor, wrapped, box, flags);
-	
 	miditrack_view->priv->real_slot_width = (box->x2 - box->x1) / miditrack_view->priv->n_slots;
 
 	if (miditrack_view->priv->show_grid){
@@ -346,7 +350,7 @@ static void internal_add_note(JammoMiditrackView* miditrack_view, gfloat width,g
 	ClutterActor* new_note_button;
 	guint slot_x, slot_y;
 	TangleWidget* widget;
-	widget = TANGLE_WIDGET(tangle_wrapper_actor_get_wrapped(TANGLE_WRAPPER_ACTOR(miditrack_view)));
+	widget = TANGLE_WIDGET(miditrack_view);
 
 	center_x -= (width - 1) * miditrack_view->priv->real_slot_width / 2;
 	slot_x = center_x / miditrack_view->priv->real_slot_width;
@@ -385,33 +389,44 @@ static void internal_add_note(JammoMiditrackView* miditrack_view, gfloat width,g
                          "y",  slot_y * miditrack_view->priv->slot_height, NULL);
 }
 
-
-static gboolean jammo_miditrack_view_handle_dropping(TangleDroppableActor* droppable_actor, TangleDragging* dragging, gboolean accepted) {
+static gboolean on_drag_end(TangleDropAction* drop_action, TangleDragAction* drag_action, gpointer user_data) {
 	JammoMiditrackView* miditrack_view;
+
+	miditrack_view = JAMMO_MIDITRACK_VIEW(user_data);
+
+	miditrack_view->priv->hilighted_start = miditrack_view->priv->hilighted_end = -1.0; //do not show hilighting anymore
+
+	return FALSE;
+}
+
+static void on_dropped(TangleDropAction* drop_action, TangleDragAction* drag_action, gpointer user_data) {
+	JammoMiditrackView* miditrack_view;
+	ClutterActor* meta_actor;
+	ClutterActor* drag_actor;
+	gfloat actor_x, actor_y, w, h;
 	gfloat x, y, center_x, center_y;
 
-	miditrack_view = JAMMO_MIDITRACK_VIEW(droppable_actor);
-		if (JAMMO_MIDITRACK_VIEW(miditrack_view)->priv->pen_mode) {
+	miditrack_view = JAMMO_MIDITRACK_VIEW(user_data);
+		if (miditrack_view->priv->pen_mode) {
 		//printf("dropping: in pen-mode\n");
 		JAMMO_MIDITRACK_VIEW(miditrack_view)->priv->discard_next_click = TRUE;
 	}
 	
-	TANGLE_DROPPABLE_ACTOR_CLASS(jammo_miditrack_view_parent_class)->handle_dropping(droppable_actor, dragging, accepted);
+	meta_actor = clutter_actor_meta_get_actor(CLUTTER_ACTOR_META(drag_action));
+	drag_actor = tangle_drag_action_get_drag_actor(drag_action);
+	clutter_actor_get_transformed_position(drag_actor, &actor_x, &actor_y);
+	clutter_actor_get_size(drag_actor, &w, &h);
+	if (clutter_actor_transform_stage_point(CLUTTER_ACTOR(miditrack_view), actor_x, actor_y, &x, &y) &&
+	    clutter_actor_transform_stage_point(CLUTTER_ACTOR(miditrack_view), actor_x + w / 2, actor_y + h / 2, &center_x, &center_y)) {
 
-	if (accepted && clutter_actor_transform_stage_point(CLUTTER_ACTOR(miditrack_view), dragging->x, dragging->y, &x, &y) &&
-	    clutter_actor_transform_stage_point(CLUTTER_ACTOR(miditrack_view), dragging->center_x, dragging->center_y, &center_x, &center_y)) {
-
-	gfloat width = (clutter_actor_get_width(CLUTTER_ACTOR(dragging->draggable_actor)) / miditrack_view->priv->real_slot_width );
+	gfloat width = (clutter_actor_get_width(clutter_actor_meta_get_actor(CLUTTER_ACTOR_META(drag_action))) / miditrack_view->priv->real_slot_width );
 	printf("handle_dropping:  %f ; %f ; %f ;%f\n",x, y, center_x, center_y);
-	internal_add_note(miditrack_view, width,x - dragging->anchor_x,y - dragging->anchor_y,center_x,center_y);
+	internal_add_note(miditrack_view, width,x - actor_x + w / 2,y - actor_y + h / 2,center_x,center_y);
 	}
 	else {
 	//printf("dropping out of bounds -> this will be deleted!\n");
-	JAMMO_MIDITRACK_VIEW(miditrack_view)->priv->discard_next_click = FALSE;
+	miditrack_view->priv->discard_next_click = FALSE;
 	}
-
-	miditrack_view->priv->hilighted_start = miditrack_view->priv->hilighted_end = -1.0; //do not show hilighting anymore
-	return FALSE;
 }
 
 static gboolean jammo_miditrack_view_handle_press(ClutterActor* clutter_actor, ClutterButtonEvent* event) {
@@ -495,26 +510,24 @@ static void jammo_miditrack_view_paint(ClutterActor* actor) {
 	}
 }
 
-static GObject* jammo_miditrack_view_constructor(GType type, guint n_construct_properties, GObjectConstructParam* construct_properties) {
-	//printf("constructor\n");
-	GObject* object;
-	JammoTrack* miditrack;
-	ClutterActor* widget;
+static void jammo_miditrack_view_constructed(GObject* object) {
+	JammoMiditrackView* miditrack_view;
+	ClutterAction* drop_action;
 
-	g_return_val_if_fail(tangle_lookup_construct_properties(type, n_construct_properties, construct_properties, "track", &miditrack, NULL), NULL);
-	g_return_val_if_fail(miditrack != NULL, NULL);
+	miditrack_view = JAMMO_MIDITRACK_VIEW(object);
 
-	widget = tangle_widget_new();
-
-	object = tangle_construct_with_extra_properties(G_OBJECT_CLASS(jammo_miditrack_view_parent_class)->constructor, type, n_construct_properties, construct_properties, "wrapped", widget, NULL);
-
-	g_signal_connect(widget, "actor-added", G_CALLBACK(on_actor_added), object);
-	g_signal_connect(widget, "actor-removed", G_CALLBACK(on_actor_removed), object);
+	g_signal_connect(miditrack_view, "actor-added", G_CALLBACK(on_actor_added), NULL);
+	g_signal_connect(miditrack_view, "actor-removed", G_CALLBACK(on_actor_removed), NULL);
 
 	JAMMO_MIDITRACK_VIEW(object)->priv->grid = TANGLE_GRID(tangle_grid_new(0.0, 0.0));
-	tangle_widget_set_background_actor(TANGLE_WIDGET(tangle_wrapper_actor_get_wrapped(TANGLE_WRAPPER_ACTOR(object))), CLUTTER_ACTOR(JAMMO_MIDITRACK_VIEW(object)->priv->grid));
+	tangle_widget_set_background_actor(TANGLE_WIDGET(miditrack_view), CLUTTER_ACTOR(JAMMO_MIDITRACK_VIEW(object)->priv->grid));
 
-	return object;
+	drop_action = tangle_drop_action_new();
+	g_signal_connect(drop_action, "drag-begin", G_CALLBACK(on_drag_begin_or_motion), miditrack_view);
+	g_signal_connect(drop_action, "drag-motion", G_CALLBACK(on_drag_begin_or_motion), miditrack_view);
+	g_signal_connect(drop_action, "drag-end", G_CALLBACK(on_drag_end), miditrack_view);
+	g_signal_connect(drop_action, "dropped", G_CALLBACK(on_dropped), miditrack_view);
+	clutter_actor_add_action(CLUTTER_ACTOR(miditrack_view), drop_action);
 }
 
 static void jammo_miditrack_view_set_property(GObject* object, guint prop_id, const GValue* value, GParamSpec* pspec) {
@@ -625,26 +638,19 @@ static void jammo_miditrack_view_class_init(JammoMiditrackViewClass* miditrack_v
 	GObjectClass* gobject_class = G_OBJECT_CLASS(miditrack_view_class);
 	ClutterActorClass* clutter_actor_class = CLUTTER_ACTOR_CLASS(miditrack_view_class);
 	TangleActorClass* actor_class = TANGLE_ACTOR_CLASS(miditrack_view_class);
-	TangleWrapperActorClass* wrapper_actor_class = TANGLE_WRAPPER_ACTOR_CLASS(miditrack_view_class);
-	TangleDroppableActorClass* droppable_actor_class = TANGLE_DROPPABLE_ACTOR_CLASS(miditrack_view_class);
 
-	gobject_class->constructor = jammo_miditrack_view_constructor;
+	gobject_class->constructed = jammo_miditrack_view_constructed;
 	gobject_class->finalize = jammo_miditrack_view_finalize;
 	gobject_class->dispose = jammo_miditrack_view_dispose;
 	gobject_class->set_property = jammo_miditrack_view_set_property;
 	gobject_class->get_property = jammo_miditrack_view_get_property;
 
 	clutter_actor_class->paint = jammo_miditrack_view_paint;
+	clutter_actor_class->allocate = jammo_miditrack_view_allocate;
 
 	actor_class->get_preferred_width = jammo_miditrack_view_get_preferred_width;
 	actor_class->get_preferred_height = jammo_miditrack_view_get_preferred_height;
 
-	wrapper_actor_class->allocate_wrapped = jammo_miditrack_view_allocate_wrapped;
-
-	droppable_actor_class->is_dragging_droppable = jammo_miditrack_view_is_dragging_droppable;
-	droppable_actor_class->handle_dragging = jammo_miditrack_view_handle_dragging;
-	droppable_actor_class->handle_dropping = jammo_miditrack_view_handle_dropping;
-
 	clutter_actor_class->button_press_event = jammo_miditrack_view_handle_press;
 	clutter_actor_class->button_release_event = jammo_miditrack_view_handle_release;
 	/**
@@ -795,20 +801,21 @@ static void on_actor_added(TangleWidget* widget, ClutterActor* actor, gpointer u
 
 	g_return_if_fail(JAMMO_IS_NOTE_BUTTON(actor));
 	
-	miditrack_view = JAMMO_MIDITRACK_VIEW(user_data);
+	miditrack_view = JAMMO_MIDITRACK_VIEW(widget);
 	note_button = JAMMO_NOTE_BUTTON(actor);
 */
 }
 
 static void on_actor_removed(TangleWidget* widget, ClutterActor* actor, gpointer user_data) {
+	//printf("on_actor_removed triggered\n");
 	JammoMiditrackView* miditrack_view;
 	JammoNoteButton* note_button;
 
 	g_return_if_fail(JAMMO_IS_NOTE_BUTTON(actor));
 	note_button=JAMMO_NOTE_BUTTON(actor);
 
-	g_return_if_fail(JAMMO_IS_MIDITRACK_VIEW(user_data));
-	miditrack_view=JAMMO_MIDITRACK_VIEW(user_data);
+	g_return_if_fail(JAMMO_IS_MIDITRACK_VIEW(widget));
+	miditrack_view=JAMMO_MIDITRACK_VIEW(widget);
 
 	JammoMidiEvent* event;
 
@@ -818,3 +825,12 @@ static void on_actor_removed(TangleWidget* widget, ClutterActor* actor, gpointer
 	jammo_instrument_track_remove_event(miditrack_view->priv->miditrack, event->note,  event->timestamp);
 }
 
+void jammo_miditrack_view_remove_all(JammoMiditrackView* miditrack_view) {
+	printf("jammo_miditrack_view_remove_all\n");
+	TangleWidget* widget = TANGLE_WIDGET(miditrack_view);
+	GList* children = clutter_container_get_children(CLUTTER_CONTAINER(widget));
+	if (children) {
+		g_list_foreach(children, (GFunc)clutter_actor_destroy, NULL);
+		g_list_free(children);
+	}
+}
