@@ -15,7 +15,7 @@
 #include <string.h>
 #include "../meam/jammo-midi.h"
 
-G_DEFINE_TYPE(JammoNoteButton, jammo_note_button, TANGLE_TYPE_DRAGGABLE_ACTOR);
+G_DEFINE_TYPE(JammoNoteButton, jammo_note_button, TANGLE_TYPE_BUTTON);
 
 enum {
 	PROP_0,
@@ -26,68 +26,102 @@ enum {
 struct _JammoNoteButtonPrivate {
 	JammoMidiEvent* start_event;
 	JammoMidiEvent* stop_event;
+	ClutterAction* drag_action;
 };
 
 static gboolean on_clicked(TangleButton* button, gpointer user_data);
 
 static const ClutterColor default_note_color = { 255, 0, 0, 128 };
+static const ClutterColor default_handle_color = { 255, 255, 0, 128 };
 
 //This is only for debugging
 ClutterActor* jammo_note_button_new_without_events() {
 	ClutterActor* note =  CLUTTER_ACTOR(g_object_new(JAMMO_TYPE_NOTE_BUTTON, NULL));
 	ClutterActor* box =  clutter_rectangle_new_with_color(&default_note_color);
 
-	tangle_button_set_normal_background_actor(TANGLE_BUTTON(tangle_wrapper_actor_get_wrapped(TANGLE_WRAPPER_ACTOR(note))), box);
+	tangle_button_set_normal_background_actor(TANGLE_BUTTON(note), box);
 	return note;
 }
 
 /*
 width,height
 and start+stop events
-*/
-ClutterActor* jammo_note_button_new(guint width, guint height, JammoMidiEvent* start, JammoMidiEvent* stop) {
-	//printf("making note with x=%d and y=%d\n",x,y);
-	ClutterActor* note =  CLUTTER_ACTOR(g_object_new(JAMMO_TYPE_NOTE_BUTTON, "start-event",start,"stop-event",stop,NULL));
-	ClutterActor* box =  clutter_rectangle_new_with_color(&default_note_color);
-	clutter_actor_set_size(box,(gfloat)width,(gfloat)height);
-	tangle_button_set_normal_background_actor(TANGLE_BUTTON(tangle_wrapper_actor_get_wrapped(TANGLE_WRAPPER_ACTOR(note))), box);
 
-	return note;
+'note_itself' is tangle_button.
+It's background-actor will be container containing 'box' and 'handle'.
+Container has layout, so handle will be always right side of the box.
+*/
+ClutterActor* jammo_note_button_new(gfloat width, gfloat height, JammoMidiEvent* start, JammoMidiEvent* stop) {
+	//printf("making note with width=%f and height=%f\n",width,height);
+	ClutterActor* note_itself =  CLUTTER_ACTOR(g_object_new(JAMMO_TYPE_NOTE_BUTTON, "start-event",start,"stop-event",stop,NULL));
+	ClutterActor* widget = tangle_widget_new();
+
+	ClutterActor* box =  clutter_rectangle_new_with_color(&default_note_color);
+	ClutterActor* handle =  clutter_rectangle_new_with_color(&default_handle_color);
+
+	ClutterAction* action;
+	TangleBinding* binding;
+	GValue value = { 0 };
+
+	gfloat handler_width = width/5;  //TODO. We can't use constant because width can be (let say) 1.0px when zoomed-out.
+	clutter_actor_set_size(box,(gfloat)width-handler_width,(gfloat)height);
+	clutter_actor_set_size(handle,handler_width,(gfloat)height);
+
+	tangle_widget_set_layout(TANGLE_WIDGET(widget), tangle_box_layout_new());
+
+	tangle_widget_add(TANGLE_WIDGET(widget), box,NULL);
+	tangle_widget_add(TANGLE_WIDGET(widget), handle,NULL);
+
+	tangle_button_set_normal_background_actor(TANGLE_BUTTON(note_itself), widget);
+
+	binding = tangle_binding_new(G_OBJECT(box), "width", G_OBJECT(handle), "x");
+	g_value_init(&value, G_TYPE_FLOAT);
+	g_value_set_float(&value, 0.0);
+	tangle_binding_set_constant_term(binding, &value);
+	g_value_unset(&value);
+
+	//This (changin height) is not used, only for testing!
+	binding = tangle_binding_new(G_OBJECT(box), "height", G_OBJECT(handle), "y");
+	g_value_init(&value, G_TYPE_FLOAT);
+	g_value_set_float(&value, -height);
+	tangle_binding_set_constant_term(binding, &value);
+	g_value_unset(&value);
+
+	action = tangle_stretch_action_new(box);
+	//tangle_stretch_action_set_min_size(TANGLE_STRETCH_ACTION(action), 100.0, 50.0);
+	//tangle_stretch_action_set_max_size(TANGLE_STRETCH_ACTION(action), 700.0, 450.0);
+	clutter_actor_add_action(handle, action);
+	clutter_actor_set_reactive(handle, TRUE);
+
+	return note_itself;
 }
 
 
 
-static gboolean jammo_note_button_end_dragging(TangleDraggableActor* draggable_actor, TangleDragging* dragging) {
-	gboolean retvalue = FALSE;
+static void on_drag_end(ClutterDragAction* action, ClutterActor* actor, gfloat event_x, gfloat event_y, ClutterModifierType modifiers, gpointer user_data) {
 	JammoNoteButton* note_button;
 	ClutterActor* parent;
-	ClutterActor* grand_parent;
 	
-	note_button = JAMMO_NOTE_BUTTON(draggable_actor);
+	note_button = JAMMO_NOTE_BUTTON(user_data);
 	
 	parent = clutter_actor_get_parent(CLUTTER_ACTOR(note_button));
-	grand_parent = clutter_actor_get_parent(parent);
-	if (JAMMO_IS_MIDITRACK_VIEW(grand_parent)) {
+	if (JAMMO_IS_MIDITRACK_VIEW(parent)) {
 		clutter_container_remove_actor(CLUTTER_CONTAINER(parent), CLUTTER_ACTOR(note_button));
-	} else {
-		retvalue = TANGLE_DRAGGABLE_ACTOR_CLASS(jammo_note_button_parent_class)->end_dragging(draggable_actor, dragging);
 	}
-	
-	return retvalue;
 }
 
-static GObject* jammo_note_button_constructor(GType type, guint n_construct_properties, GObjectConstructParam* construct_properties) {
-	GObject* object;
-	ClutterActor* button;
+static void jammo_note_button_constructed(GObject* object) {
+	JammoNoteButton* note_button;
+	
+	note_button = JAMMO_NOTE_BUTTON(object);
 
-	button = tangle_button_new();
-	tangle_widget_set_prefer_background_size(TANGLE_WIDGET(button), TRUE);
+	note_button->priv->drag_action = tangle_drag_action_new();
+	g_signal_connect(note_button->priv->drag_action, "drag-end", G_CALLBACK(on_drag_end), note_button);
+	clutter_actor_add_action(CLUTTER_ACTOR(object), note_button->priv->drag_action);
 
-	object = tangle_construct_with_extra_properties(G_OBJECT_CLASS(jammo_note_button_parent_class)->constructor, type, n_construct_properties, construct_properties, "wrapped", button, NULL);
+	tangle_widget_set_prefer_background_size(TANGLE_WIDGET(note_button), TRUE);
 
-	g_signal_connect(button, "clicked", G_CALLBACK(on_clicked), object);
-
-	return object;
+	g_signal_connect(note_button, "clicked", G_CALLBACK(on_clicked), NULL);
 }
 
 static void jammo_note_button_set_property(GObject* object, guint prop_id, const GValue* value, GParamSpec* pspec) {
@@ -147,16 +181,13 @@ static void jammo_note_button_dispose(GObject* object) {
 
 static void jammo_note_button_class_init(JammoNoteButtonClass* note_button_class) {
 	GObjectClass* gobject_class = G_OBJECT_CLASS(note_button_class);
-	TangleDraggableActorClass* draggable_actor_class = TANGLE_DRAGGABLE_ACTOR_CLASS(note_button_class);
 
-	gobject_class->constructor = jammo_note_button_constructor;
+	gobject_class->constructed = jammo_note_button_constructed;
 	gobject_class->finalize = jammo_note_button_finalize;
 	gobject_class->dispose = jammo_note_button_dispose;
 	gobject_class->set_property = jammo_note_button_set_property;
 	gobject_class->get_property = jammo_note_button_get_property;
 
-	draggable_actor_class->end_dragging = jammo_note_button_end_dragging;
-
 	/**
 	 * JammoNoteButton:start-event:
 	 */
@@ -188,7 +219,7 @@ static void jammo_note_button_init(JammoNoteButton* note_button) {
 
 static gboolean on_clicked(TangleButton* button, gpointer user_data) {
 	JammoNoteButton* note_button;
-	note_button = JAMMO_NOTE_BUTTON(user_data);
+	note_button = JAMMO_NOTE_BUTTON(button);
 
 	GstClockTime start=0;
 	GstClockTime stop=0;
