@@ -12,7 +12,7 @@
 #include "jammo-sample-button.h"
 #include "../meam/jammo-sample.h"
 #include "../gems/gems.h"
-G_DEFINE_TYPE(JammoTrackView, jammo_track_view, TANGLE_TYPE_DROPPABLE_ACTOR);
+G_DEFINE_TYPE(JammoTrackView, jammo_track_view, TANGLE_TYPE_WIDGET);
 
 enum {
 	PROP_0,
@@ -64,6 +64,9 @@ static const ClutterColor default_hilight_color = { 255, 255, 255, 100 };
 static void on_actor_added(TangleWidget* widget, ClutterActor* actor, gpointer user_data);
 static void on_actor_removed(TangleWidget* widget, ClutterActor* actor, gpointer user_data);
 static gboolean check_if_overlapping(JammoTrackView* track_view, ClutterActor* sample_button, guint slot_index);
+static gboolean on_drag_begin_or_motion(TangleDropAction* drop_action, TangleDragAction* drag_action, gpointer user_data);
+static gboolean on_drag_end(TangleDropAction* drop_action, TangleDragAction* drag_action, gpointer user_data);
+static void on_dropped(TangleDropAction* drop_action, TangleDragAction* drag_action, gpointer user_data);
 static void marshal_BOOLEAN__OBJECT (GClosure     *closure,
                                 GValue       *return_value G_GNUC_UNUSED,
                                 guint         n_param_values,
@@ -77,19 +80,46 @@ ClutterActor* jammo_track_view_new(JammoEditingTrack* track, guint n_slots, guin
 	return CLUTTER_ACTOR(g_object_new(JAMMO_TYPE_TRACK_VIEW, "track", track, "n-slots", n_slots, "slot-duration", slot_duration, "slot-width", slot_width, "slot-height", slot_height, NULL));
 }
 
-/* Note! This function does not check if the given sample will be overlapping some other sample. */
-void jammo_track_view_add_jammo_sample_button(JammoTrackView* track_view, JammoSampleButton* sample_button, guint slot) {
-	TangleWidget* widget;
 
-	jammo_editing_track_add_sample(track_view->priv->track, jammo_sample_button_get_sample(sample_button), slot * track_view->priv->slot_duration);
+static void on_sample_duration_notify(GObject* object, GParamSpec* pspec, gpointer user_data) {
+	printf("on_sample_duration_notify\n");
+	TangleVault* vault;
+	JammoSampleButton* sample_button;
+	JammoTrackView* track_view;
+	guint slot;
 
-	widget = TANGLE_WIDGET(tangle_wrapper_actor_get_wrapped(TANGLE_WRAPPER_ACTOR(track_view)));
+	vault = TANGLE_VAULT(user_data);
+	tangle_vault_get(vault, 3, JAMMO_TYPE_SAMPLE_BUTTON, &sample_button, JAMMO_TYPE_TRACK_VIEW, &track_view, G_TYPE_UINT, &slot);
 
 	clutter_actor_set_x(CLUTTER_ACTOR(sample_button),
-		slot * (track_view->priv->real_slot_width ? track_view->priv->real_slot_width : track_view->priv->slot_width));
+          slot * (track_view->priv->real_slot_width ? track_view->priv->real_slot_width : track_view->priv->slot_width));
 	clutter_actor_set_y(CLUTTER_ACTOR(sample_button), 0.0);
 	clutter_actor_set_height(CLUTTER_ACTOR(sample_button), track_view->priv->slot_height);
-	tangle_widget_add(TANGLE_WIDGET(widget), CLUTTER_ACTOR(sample_button), NULL);
+
+
+	guint64 duration_of_sample = jammo_sample_get_duration(jammo_sample_button_get_sample(JAMMO_SAMPLE_BUTTON(sample_button)));
+	guint n_slots = (guint) ((double)duration_of_sample / (double) track_view->priv->slot_duration + 0.5);
+	clutter_actor_set_width(CLUTTER_ACTOR(sample_button), n_slots * track_view->priv->slot_width);
+
+	//printf("duration_of_one_slot: %llu, duration_of_sample: %llu, n_slots: %d\n", duration_of_one_slot, duration_of_sample, n_slots);
+	tangle_widget_set_background_color(TANGLE_WIDGET(sample_button), &track_view->priv->sample_color);
+
+	tangle_widget_add(TANGLE_WIDGET(track_view), CLUTTER_ACTOR(sample_button), NULL);
+	g_signal_handlers_disconnect_by_func(object, G_CALLBACK(on_sample_duration_notify), user_data);
+}
+
+
+
+/* Note! This function does not check if the given sample will be overlapping some other sample. */
+void jammo_track_view_add_jammo_sample_button(JammoTrackView* track_view, JammoSampleButton* sample_button, guint slot) {
+	jammo_editing_track_add_sample(track_view->priv->track, jammo_sample_button_get_sample(sample_button), slot * track_view->priv->slot_duration);
+
+	//We need to wait until duration of sample is calculated (we need it to adjust width)
+	TangleVault* vault;
+
+	vault = tangle_vault_new(3, JAMMO_TYPE_SAMPLE_BUTTON, sample_button, JAMMO_TYPE_TRACK_VIEW, track_view, G_TYPE_UINT, slot);
+	tangle_signal_connect_vault(jammo_sample_button_get_sample(JAMMO_SAMPLE_BUTTON(sample_button)),
+           "notify::duration", G_CALLBACK(on_sample_duration_notify), vault);
 }
 
 /*
@@ -97,22 +127,17 @@ Used when removing over network,
 */
 void jammo_track_view_remove_jammo_sample_button_from_slot(JammoTrackView* track_view, guint asked_slot) {
 	gfloat asked_x = asked_slot * track_view->priv->slot_width;
-	//printf("looking sample-button from x-position '%f'\n",asked_x);
-	TangleWidget* widget;
 	GList* children;
 	ClutterActor* actor = NULL;
 
 	gfloat x;  //left side of actor
 	gfloat x2; //x2=x+width, right side of actor
-	widget = TANGLE_WIDGET(tangle_wrapper_actor_get_wrapped(TANGLE_WRAPPER_ACTOR(track_view)));
 
-	for (children = tangle_widget_get_children_readonly(widget); children; children = children->next) {
+	for (children = tangle_widget_get_children_readonly(TANGLE_WIDGET(track_view)); children; children = children->next) {
 		actor = CLUTTER_ACTOR(children->data);
 		x = clutter_actor_get_x(actor);
 		x2 = x + clutter_actor_get_width(actor);
-		//printf("this sample-button covers '%f' ... '%f'\n",x,x2);
 		if (x<= asked_x && asked_x < x2) {
-			//printf("found\n");
 			clutter_container_remove_actor(CLUTTER_CONTAINER(clutter_actor_get_parent(actor)), actor);
 			break;
 		}
@@ -120,7 +145,7 @@ void jammo_track_view_remove_jammo_sample_button_from_slot(JammoTrackView* track
 }
 
 guint jammo_track_view_get_sample_button_slot(JammoTrackView* track_view, JammoSampleButton* sample_button) {
-	g_return_val_if_fail(clutter_actor_get_parent(CLUTTER_ACTOR(sample_button)) == tangle_wrapper_actor_get_wrapped(TANGLE_WRAPPER_ACTOR(track_view)), (guint)-1);
+	g_return_val_if_fail(clutter_actor_get_parent(CLUTTER_ACTOR(sample_button)) == CLUTTER_ACTOR(track_view), (guint)-1);
 
 	return (guint)(clutter_actor_get_x(CLUTTER_ACTOR(sample_button)) / track_view->priv->real_slot_width);
 }
@@ -136,9 +161,9 @@ void jammo_track_view_set_editing_enabled(JammoTrackView* track_view, gboolean e
 	if (track_view->priv->editing_enabled != editing_enabled) {
 		track_view->priv->editing_enabled = editing_enabled;
 		
-		tangle_droppable_actor_set_dropping_enabled(TANGLE_DROPPABLE_ACTOR(track_view), editing_enabled);
-		for (children = tangle_widget_get_children_readonly(TANGLE_WIDGET(tangle_wrapper_actor_get_wrapped(TANGLE_WRAPPER_ACTOR(track_view)))); children; children = children->next) {
-			tangle_draggable_actor_set_dragging_enabled(TANGLE_DRAGGABLE_ACTOR(children->data), editing_enabled);
+		clutter_actor_meta_set_enabled(CLUTTER_ACTOR_META(tangle_actor_get_action_by_type(CLUTTER_ACTOR(track_view), TANGLE_TYPE_DROP_ACTION)), editing_enabled);
+		for (children = tangle_widget_get_children_readonly(TANGLE_WIDGET(track_view)); children; children = children->next) {
+			clutter_actor_meta_set_enabled(CLUTTER_ACTOR_META(tangle_actor_get_action_by_type(CLUTTER_ACTOR(children->data), TANGLE_TYPE_DRAG_ACTION)), editing_enabled);
 		}
 		
 		g_object_notify(G_OBJECT(track_view), "editing-enabled");
@@ -157,53 +182,6 @@ void jammo_track_view_set_sample_type(JammoTrackView* track_view, JammoSampleTyp
 	}
 }
 
-static gboolean jammo_track_view_is_dragging_droppable(TangleDroppableActor* droppable_actor, TangleDragging* dragging) {
-	gboolean droppable = FALSE;
-	JammoTrackView* track_view;
-	gfloat center_x, center_y;
-	guint slot = 0;
-	guint n_slots = 0;
-	gfloat start;
-	
-	track_view = JAMMO_TRACK_VIEW(droppable_actor);
-	
-	if (JAMMO_IS_SAMPLE_BUTTON(dragging->draggable_actor) &&
-	    clutter_actor_transform_stage_point(CLUTTER_ACTOR(track_view), dragging->center_x, dragging->center_y, &center_x, &center_y)) {
-
-		guint64 duration_of_one_slot= track_view->priv->slot_duration;
-		guint64 duration_of_sample = jammo_sample_get_duration(jammo_sample_button_get_sample(JAMMO_SAMPLE_BUTTON(dragging->draggable_actor)));
-		n_slots = (guint) ((double)duration_of_sample / (double) duration_of_one_slot + 0.5);
-
-		center_x -= (n_slots - 1) * track_view->priv->real_slot_width / 2;
-		slot = center_x / track_view->priv->real_slot_width;
-
-		if (slot >= track_view->priv->disabled_slots_begin &&
-		    slot + n_slots - 1 < track_view->priv->n_slots - track_view->priv->disabled_slots_end &&
-		    !check_if_overlapping(track_view, CLUTTER_ACTOR(dragging->draggable_actor), slot)) {
-			droppable = TRUE;
-		}
-	}
-	
-	if (droppable) {
-		start = slot * track_view->priv->real_slot_width;
-		if (start != track_view->priv->hilighted_start) {
-			track_view->priv->hilighted_start = start;
-			track_view->priv->hilighted_end = start + n_slots * track_view->priv->real_slot_width;
-			clutter_actor_queue_redraw(CLUTTER_ACTOR(track_view));
-		}
-	} else if (track_view->priv->hilighted_start != track_view->priv->hilighted_end) {
-		track_view->priv->hilighted_start = track_view->priv->hilighted_end = 0.0;
-		clutter_actor_queue_redraw(CLUTTER_ACTOR(track_view));
-	}
-	    		
-	return droppable;
-}
-
-static gboolean jammo_track_view_handle_dragging(TangleDroppableActor* droppable_actor, TangleDragging* dragging) {
-
-	return FALSE;
-}
-
 static void jammo_track_view_get_preferred_width(TangleActor* actor, gfloat for_height, gboolean interacting, gfloat* min_width_p, gfloat* natural_width_p, gfloat* max_width_p) {
 	JammoTrackView* track_view;
 	
@@ -236,12 +214,12 @@ static void jammo_track_view_get_preferred_height(TangleActor* actor, gfloat for
 	}
 }
 
-static void jammo_track_view_allocate_wrapped(TangleWrapperActor* wrapper_actor, ClutterActor* wrapped, const ClutterActorBox* box, ClutterAllocationFlags flags) {
+static void jammo_track_view_allocate(ClutterActor* actor, const ClutterActorBox* box, ClutterAllocationFlags flags) {
 	JammoTrackView* track_view;
 
-	track_view = JAMMO_TRACK_VIEW(wrapper_actor);
+	track_view = JAMMO_TRACK_VIEW(actor);
 
-	TANGLE_WRAPPER_ACTOR_CLASS(jammo_track_view_parent_class)->allocate_wrapped(wrapper_actor, wrapped, box, flags);
+	CLUTTER_ACTOR_CLASS(jammo_track_view_parent_class)->allocate(actor, box, flags);
 	
 	track_view->priv->real_slot_width = (box->x2 - box->x1) / track_view->priv->n_slots;
 
@@ -254,64 +232,6 @@ static void jammo_track_view_allocate_wrapped(TangleWrapperActor* wrapper_actor,
 	}
 }
 
-static gboolean jammo_track_view_handle_dropping(TangleDroppableActor* droppable_actor, TangleDragging* dragging, gboolean accepted) {
-	JammoTrackView* track_view;
-	TangleWidget* widget;
-	gfloat x, y, center_x, center_y;
-	JammoSampleButton* sample_button;
-	ClutterActor* copy_of_sample_button;
-	guint slot;
-	guint n_slots;
-	guint64 duration_of_one_slot;
-	guint64 duration_of_sample;
-
-	track_view = JAMMO_TRACK_VIEW(droppable_actor);
-	widget = TANGLE_WIDGET(tangle_wrapper_actor_get_wrapped(TANGLE_WRAPPER_ACTOR(track_view)));
-	
-	TANGLE_DROPPABLE_ACTOR_CLASS(jammo_track_view_parent_class)->handle_dropping(droppable_actor, dragging, accepted);
-
-	if (accepted && clutter_actor_transform_stage_point(CLUTTER_ACTOR(track_view), dragging->x, dragging->y, &x, &y) &&
-	    clutter_actor_transform_stage_point(CLUTTER_ACTOR(track_view), dragging->center_x, dragging->center_y, &center_x, &center_y)) {
-		if (track_view->priv->sample_type != JAMMO_SAMPLE_UNTYPED &&
-		     jammo_sample_button_get_sample_type(JAMMO_SAMPLE_BUTTON(dragging->draggable_actor)) != track_view->priv->sample_type &&
-		     jammo_sample_button_get_sample_type(JAMMO_SAMPLE_BUTTON(dragging->draggable_actor)) != JAMMO_SAMPLE_UNTYPED) {
-		     	accepted = FALSE;
-			g_signal_emit(track_view, signals[DROPPING_INCOMPATIBLE_SAMPLE], 0, dragging->draggable_actor, &accepted);
-		}
-	}
-
-	if (accepted) {
-		sample_button = JAMMO_SAMPLE_BUTTON(dragging->draggable_actor);
-		copy_of_sample_button = jammo_sample_button_new_from_existing(sample_button);
-
-		jammo_sample_button_set_background_color(JAMMO_SAMPLE_BUTTON(copy_of_sample_button), &track_view->priv->sample_color);
-
-		duration_of_one_slot = track_view->priv->slot_duration;
-		duration_of_sample = jammo_sample_get_duration(jammo_sample_button_get_sample(JAMMO_SAMPLE_BUTTON(dragging->draggable_actor)));
-		n_slots = (guint) ((double)duration_of_sample / (double) duration_of_one_slot + 0.5);
-
-		center_x -= (n_slots - 1) * track_view->priv->real_slot_width / 2;
-		slot = center_x / track_view->priv->real_slot_width;
-
-		jammo_editing_track_add_sample(track_view->priv->track, jammo_sample_button_get_sample(JAMMO_SAMPLE_BUTTON(copy_of_sample_button)), slot * track_view->priv->slot_duration);
-
-		/* tangle_draggable_actor_set_floating_mode_dragged(TANGLE_DRAGGABLE_ACTOR(copy_of_sample_button), TANGLE_FLOATING_MODE_COLLAPSE); */
-		clutter_actor_set_x(copy_of_sample_button, x - dragging->anchor_x);
-		clutter_actor_set_y(copy_of_sample_button, y - dragging->anchor_y);
-		clutter_actor_set_height(copy_of_sample_button, track_view->priv->slot_height);
-		clutter_actor_set_width(copy_of_sample_button, n_slots * track_view->priv->slot_width);
-		tangle_widget_add(widget, copy_of_sample_button, NULL);
-
-		clutter_actor_animate(copy_of_sample_button, CLUTTER_EASE_OUT_CUBIC, 300, "x", slot * track_view->priv->real_slot_width, "y", 0.0, NULL);
-
-		gems_add_new_sample_to_track(jammo_sample_button_get_loop_id(sample_button), slot);
-	}
-	
-	track_view->priv->hilighted_start = track_view->priv->hilighted_end = -1.0;
-
-	return FALSE;
-}
-
 static void jammo_track_view_paint(ClutterActor* actor) {
 	JammoTrackView* track_view;
 	gdouble scale_x;
@@ -343,25 +263,24 @@ static void jammo_track_view_paint(ClutterActor* actor) {
 	}
 }
 
-static GObject* jammo_track_view_constructor(GType type, guint n_construct_properties, GObjectConstructParam* construct_properties) {
-	GObject* object;
-	JammoTrack* track;
-	ClutterActor* widget;
+static void jammo_track_view_constructed(GObject* object) {
+	JammoTrackView* track_view;
+	ClutterAction* drop_action;
 
-	g_return_val_if_fail(tangle_lookup_construct_properties(type, n_construct_properties, construct_properties, "track", &track, NULL), NULL);
-	g_return_val_if_fail(track != NULL, NULL);
-
-	widget = tangle_widget_new();
-
-	object = tangle_construct_with_extra_properties(G_OBJECT_CLASS(jammo_track_view_parent_class)->constructor, type, n_construct_properties, construct_properties, "wrapped", widget, NULL);
-
-	g_signal_connect(widget, "actor-added", G_CALLBACK(on_actor_added), object);
-	g_signal_connect(widget, "actor-removed", G_CALLBACK(on_actor_removed), object);
+	track_view = JAMMO_TRACK_VIEW(object);
 	
-	JAMMO_TRACK_VIEW(object)->priv->grid = TANGLE_GRID(tangle_grid_new(0.0, 0.0));
-	tangle_widget_set_background_actor(TANGLE_WIDGET(tangle_wrapper_actor_get_wrapped(TANGLE_WRAPPER_ACTOR(object))), CLUTTER_ACTOR(JAMMO_TRACK_VIEW(object)->priv->grid));
+	g_signal_connect(track_view, "actor-added", G_CALLBACK(on_actor_added), NULL);
+	g_signal_connect(track_view, "actor-removed", G_CALLBACK(on_actor_removed), NULL);
+	
+	track_view->priv->grid = TANGLE_GRID(tangle_grid_new(0.0, 0.0));
+	tangle_widget_set_background_actor(TANGLE_WIDGET(object), CLUTTER_ACTOR(track_view->priv->grid));
 
-	return object;
+	drop_action = tangle_drop_action_new();
+	g_signal_connect(drop_action, "drag-begin", G_CALLBACK(on_drag_begin_or_motion), track_view);
+	g_signal_connect(drop_action, "drag-motion", G_CALLBACK(on_drag_begin_or_motion), track_view);
+	g_signal_connect(drop_action, "drag-end", G_CALLBACK(on_drag_end), track_view);
+	g_signal_connect(drop_action, "dropped", G_CALLBACK(on_dropped), track_view);
+	clutter_actor_add_action(CLUTTER_ACTOR(track_view), drop_action);
 }
 
 static void jammo_track_view_set_property(GObject* object, guint prop_id, const GValue* value, GParamSpec* pspec) {
@@ -476,26 +395,19 @@ static void jammo_track_view_class_init(JammoTrackViewClass* track_view_class) {
 	GObjectClass* gobject_class = G_OBJECT_CLASS(track_view_class);
 	ClutterActorClass* clutter_actor_class = CLUTTER_ACTOR_CLASS(track_view_class);
 	TangleActorClass* actor_class = TANGLE_ACTOR_CLASS(track_view_class);
-	TangleWrapperActorClass* wrapper_actor_class = TANGLE_WRAPPER_ACTOR_CLASS(track_view_class);
-	TangleDroppableActorClass* droppable_actor_class = TANGLE_DROPPABLE_ACTOR_CLASS(track_view_class);
 
-	gobject_class->constructor = jammo_track_view_constructor;
+	gobject_class->constructed = jammo_track_view_constructed;
 	gobject_class->finalize = jammo_track_view_finalize;
 	gobject_class->dispose = jammo_track_view_dispose;
 	gobject_class->set_property = jammo_track_view_set_property;
 	gobject_class->get_property = jammo_track_view_get_property;
 	
 	clutter_actor_class->paint = jammo_track_view_paint;
+	clutter_actor_class->allocate = jammo_track_view_allocate;
 	
 	actor_class->get_preferred_width = jammo_track_view_get_preferred_width;
 	actor_class->get_preferred_height = jammo_track_view_get_preferred_height;
 	
-	wrapper_actor_class->allocate_wrapped = jammo_track_view_allocate_wrapped;
-	
-	droppable_actor_class->is_dragging_droppable = jammo_track_view_is_dragging_droppable;
-	droppable_actor_class->handle_dragging = jammo_track_view_handle_dragging;
-	droppable_actor_class->handle_dropping = jammo_track_view_handle_dropping;
-
 	/**
 	 * JammoTrackView:track:
 	 *
@@ -615,7 +527,7 @@ static void jammo_track_view_class_init(JammoTrackViewClass* track_view_class) {
 	                                clutter_param_spec_color("hilight-color",
 	                                "Hilight color",
 	                                "A color that is used to hilight selected slots when dragging samples",
-	                                &default_sample_color,
+	                                &default_hilight_color,
 	                                G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |G_PARAM_STATIC_BLURB));
 	/**
 	 * JammoTrackView:sample-type:
@@ -669,7 +581,7 @@ static void on_actor_added(TangleWidget* widget, ClutterActor* actor, gpointer u
 
 	g_return_if_fail(JAMMO_IS_SAMPLE_BUTTON(actor));
 	
-	track_view = JAMMO_TRACK_VIEW(user_data);
+	track_view = JAMMO_TRACK_VIEW(widget);
 	sample_button = JAMMO_SAMPLE_BUTTON(actor);
 	sample = jammo_sample_button_get_sample(sample_button);
 	
@@ -682,7 +594,7 @@ static void on_actor_removed(TangleWidget* widget, ClutterActor* actor, gpointer
 
 	g_return_if_fail(JAMMO_IS_SAMPLE_BUTTON(actor));
 	
-	track_view = JAMMO_TRACK_VIEW(user_data);
+	track_view = JAMMO_TRACK_VIEW(widget);
 	sample_button = JAMMO_SAMPLE_BUTTON(actor);
 	
 	jammo_editing_track_remove_sample(track_view->priv->track, jammo_sample_button_get_sample(sample_button));
@@ -696,18 +608,16 @@ static void on_actor_removed(TangleWidget* widget, ClutterActor* actor, gpointer
 }
 
 static gboolean check_if_overlapping(JammoTrackView* track_view, ClutterActor* sample_button, guint slot_index) {
-	TangleWidget* widget;
 	guint n_slots;
 	GList* children;
 	ClutterActor* actor = NULL;
 	gfloat x;
 	
-	widget = TANGLE_WIDGET(tangle_wrapper_actor_get_wrapped(TANGLE_WRAPPER_ACTOR(track_view)));
 	guint64 duration_of_one_slot= track_view->priv->slot_duration;
 	guint64 duration_of_sample = jammo_sample_get_duration(jammo_sample_button_get_sample(JAMMO_SAMPLE_BUTTON(sample_button)));
 	n_slots = (guint) ((double)duration_of_sample / (double) duration_of_one_slot + 0.5);
 
-	for (children = tangle_widget_get_children_readonly(widget); children; children = children->next) {
+	for (children = tangle_widget_get_children_readonly(TANGLE_WIDGET(track_view)); children; children = children->next) {
 		actor = CLUTTER_ACTOR(children->data);
 		x = clutter_actor_get_x(actor);
 		if (actor != sample_button &&
@@ -720,7 +630,7 @@ static gboolean check_if_overlapping(JammoTrackView* track_view, ClutterActor* s
 		actor = NULL;
 	}
 	if (!actor) {
-		for (children = tangle_widget_get_children_readonly(widget); children; children = children->next) {
+		for (children = tangle_widget_get_children_readonly(TANGLE_WIDGET(track_view)); children; children = children->next) {
 			actor = CLUTTER_ACTOR(children->data);
 			x = clutter_actor_get_x(actor);
 			if (actor != sample_button &&
@@ -736,6 +646,126 @@ static gboolean check_if_overlapping(JammoTrackView* track_view, ClutterActor* s
 	return actor != NULL;
 }
 
+static gboolean on_drag_begin_or_motion(TangleDropAction* drop_action, TangleDragAction* drag_action, gpointer user_data) {
+	gboolean droppable = FALSE;
+	JammoTrackView* track_view;
+	ClutterActor* meta_actor;
+	ClutterActor* drag_actor;
+	gfloat x, y, w, h;
+	gfloat center_x, center_y;
+	guint slot = 0;
+	guint n_slots = 0;
+	gfloat start;
+
+	track_view = JAMMO_TRACK_VIEW(user_data);
+	
+	meta_actor = clutter_actor_meta_get_actor(CLUTTER_ACTOR_META(drag_action));
+	drag_actor = tangle_drag_action_get_drag_actor(drag_action);;
+	if (JAMMO_IS_SAMPLE_BUTTON(meta_actor)) {
+		clutter_actor_get_transformed_position(drag_actor, &x, &y);
+		clutter_actor_get_size(drag_actor, &w, &h);
+		if (clutter_actor_transform_stage_point(CLUTTER_ACTOR(track_view), x + w / 2, y + h / 2, &center_x, &center_y)) {
+
+			guint64 duration_of_one_slot= track_view->priv->slot_duration;
+			guint64 duration_of_sample = jammo_sample_get_duration(jammo_sample_button_get_sample(JAMMO_SAMPLE_BUTTON(meta_actor)));
+			n_slots = (guint) ((double)duration_of_sample / (double) duration_of_one_slot + 0.5);
+
+			center_x -= (n_slots - 1) * track_view->priv->real_slot_width / 2;
+			slot = center_x / track_view->priv->real_slot_width;
+
+			if (slot >= track_view->priv->disabled_slots_begin &&
+			    slot + n_slots - 1 < track_view->priv->n_slots - track_view->priv->disabled_slots_end &&
+			    !check_if_overlapping(track_view, meta_actor, slot)) {
+				droppable = TRUE;
+			}
+		}
+	}
+	
+	if (droppable) {
+		start = slot * track_view->priv->real_slot_width;
+		if (start != track_view->priv->hilighted_start) {
+			track_view->priv->hilighted_start = start;
+			track_view->priv->hilighted_end = start + n_slots * track_view->priv->real_slot_width;
+			clutter_actor_queue_redraw(CLUTTER_ACTOR(track_view));
+		}
+	} else if (track_view->priv->hilighted_start != track_view->priv->hilighted_end) {
+		track_view->priv->hilighted_start = track_view->priv->hilighted_end = 0.0;
+		clutter_actor_queue_redraw(CLUTTER_ACTOR(track_view));
+	}
+	    		
+	return !droppable;
+}
+
+static gboolean on_drag_end(TangleDropAction* drop_action, TangleDragAction* drag_action, gpointer user_data) {	
+	JammoTrackView* track_view;
+
+	track_view = JAMMO_TRACK_VIEW(user_data);
+
+	track_view->priv->hilighted_start = track_view->priv->hilighted_end = -1.0;
+
+	return FALSE;
+}
+
+static void on_dropped(TangleDropAction* drop_action, TangleDragAction* drag_action, gpointer user_data) {
+	JammoTrackView* track_view;
+	gboolean accepted;
+	ClutterActor* meta_actor;
+	ClutterActor* drag_actor;
+	gfloat actor_x, actor_y, w, h;
+	gfloat x, y, center_x, center_y;
+	JammoSampleButton* sample_button;
+	ClutterActor* copy_of_sample_button;
+	guint slot;
+	guint n_slots;
+	guint64 duration_of_one_slot;
+	guint64 duration_of_sample;
+
+	track_view = JAMMO_TRACK_VIEW(user_data);
+	
+	accepted = TRUE;
+	meta_actor = clutter_actor_meta_get_actor(CLUTTER_ACTOR_META(drag_action));
+	drag_actor = tangle_drag_action_get_drag_actor(drag_action);
+	clutter_actor_get_transformed_position(drag_actor, &actor_x, &actor_y);
+	clutter_actor_get_size(drag_actor, &w, &h);
+	if (clutter_actor_transform_stage_point(CLUTTER_ACTOR(track_view), actor_x, actor_y, &x, &y) &&
+	    clutter_actor_transform_stage_point(CLUTTER_ACTOR(track_view), actor_x + w / 2, actor_y + h / 2, &center_x, &center_y)) {
+		if (track_view->priv->sample_type != JAMMO_SAMPLE_UNTYPED &&
+		     jammo_sample_button_get_sample_type(JAMMO_SAMPLE_BUTTON(meta_actor)) != track_view->priv->sample_type &&
+		     jammo_sample_button_get_sample_type(JAMMO_SAMPLE_BUTTON(meta_actor)) != JAMMO_SAMPLE_UNTYPED) {
+		     	accepted = FALSE;
+			g_signal_emit(track_view, signals[DROPPING_INCOMPATIBLE_SAMPLE], 0, drag_actor, &accepted);
+		}
+	}
+
+	if (accepted) {
+		sample_button = JAMMO_SAMPLE_BUTTON(meta_actor);
+		copy_of_sample_button = jammo_sample_button_new_from_existing(sample_button);
+
+		tangle_widget_set_background_color(TANGLE_WIDGET(copy_of_sample_button), &track_view->priv->sample_color);
+
+		duration_of_one_slot = track_view->priv->slot_duration;
+		duration_of_sample = jammo_sample_get_duration(jammo_sample_button_get_sample(JAMMO_SAMPLE_BUTTON(meta_actor)));
+		n_slots = (guint) ((double)duration_of_sample / (double) duration_of_one_slot + 0.5);
+
+		center_x -= (n_slots - 1) * track_view->priv->real_slot_width / 2;
+		slot = center_x / track_view->priv->real_slot_width;
+
+		jammo_editing_track_add_sample(track_view->priv->track, jammo_sample_button_get_sample(JAMMO_SAMPLE_BUTTON(copy_of_sample_button)), slot * track_view->priv->slot_duration);
+
+		/* tangle_draggable_actor_set_floating_mode_dragged(TANGLE_DRAGGABLE_ACTOR(copy_of_sample_button), TANGLE_FLOATING_MODE_COLLAPSE); */
+		clutter_actor_set_x(copy_of_sample_button, actor_x);
+		clutter_actor_set_y(copy_of_sample_button, actor_y);
+		clutter_actor_set_height(copy_of_sample_button, track_view->priv->slot_height);
+		clutter_actor_set_width(copy_of_sample_button, n_slots * track_view->priv->slot_width);
+		tangle_widget_add(TANGLE_WIDGET(track_view), copy_of_sample_button, NULL);
+
+		clutter_actor_animate(copy_of_sample_button, CLUTTER_EASE_OUT_CUBIC, 300, "x", slot * track_view->priv->real_slot_width, "y", 0.0, NULL);
+
+		gems_add_new_sample_to_track(jammo_sample_button_get_loop_id(sample_button), slot);
+	}
+}
+
+
 /* Generated marshaler */
 
 #ifdef G_ENABLE_DEBUG
