@@ -103,7 +103,7 @@ int decode_add_new_sample_to_track(char* message,int length)//length==size of re
 	location=location+sizeof(guint64);
 
 	//call upper level function with these parameters
-	printf("\nTrack number: %d, id: %s, starttime: %lld, message length: %d\n",track_number, srcLocation, startTime, location);
+	printf("\nTrack number: %d, id: %s, starttime: %" G_GINT64_FORMAT ", message length: %d\n",track_number, srcLocation, startTime, location);
 	//chum_add_new_sample_to_track_remote(g_ntohl(track_number), srcLocation, GUINT64_FROM_BE(startTime));
 	
 	return location;
