@@ -30,7 +30,7 @@ static void empty_callback_loop_sync(GObject * game, guint user_id, GList * list
 static void empty_callback_midi_list(GObject * game, guint user_id, gint16 instrument_id, GemsEventListOperation operation, GList * list){}
 static void empty_callback_slider_event_list(GObject * game, guint user_id, gint16 instrument_id, GemsEventListOperation operation, GList * list){}
 static void empty_callback_group_control(gint type){}
-static void empty_callback_game(GObject * game, gint type){}
+static void empty_callback_game(GObject * game, gint type, const gchar* song_info){}
 
 // A "singleton" get
 gems_components* gems_get_data()
@@ -974,7 +974,7 @@ void gems_set_callback_slider_event_list(void (*func_ptr)(GObject * game, guint
 void gems_set_callback_group_control(void (*func_ptr)(gint type) ) {
 	chum_callback_group_control=func_ptr;
 }
-void gems_set_callback_game(void (*func_ptr)(GObject * game, gint type) ) {
+void gems_set_callback_game(void (*func_ptr)(GObject * game, gint type, const gchar* song_info) ) {
 	chum_callback_game=func_ptr;
 }
 
