@@ -30,6 +30,7 @@ struct _JammoEditingTrackPrivate {
 	GstElement* composition;
 	GList* samples;
 	guint tempo;
+	gchar* pitch;
 };
 
 static void link_compatible_pads(GstElement* source_element, GstPad* source_pad, GstElement* sink_element);
@@ -69,6 +70,9 @@ void jammo_editing_track_add_sample(JammoEditingTrack* editing_track, JammoSampl
 			break;
 		}
 	}
+
+	jammo_sample_set_tempo(sample, editing_track->priv->tempo);
+	jammo_sample_set_pitch(sample, editing_track->priv->pitch);
 	
 	editing_track->priv->samples = g_list_insert_before(editing_track->priv->samples, list, sample);
 	element = _jammo_sample_get_element(sample);
@@ -198,8 +202,13 @@ static void jammo_editing_track_set_pitch(JammoTrack* track, const gchar* pitch)
 	
 	editing_track = JAMMO_EDITING_TRACK(track);
 	
-	for (list = editing_track->priv->samples; list; list = list->next) {
-		jammo_sample_set_pitch(JAMMO_SAMPLE(list->data), pitch);
+	if (g_strcmp0(editing_track->priv->pitch, pitch)) {
+		g_free(editing_track->priv->pitch);
+		editing_track->priv->pitch = g_strdup(pitch);
+		
+		for (list = editing_track->priv->samples; list; list = list->next) {
+			jammo_sample_set_pitch(JAMMO_SAMPLE(list->data), pitch);
+		}
 	}
 }
 
@@ -301,6 +310,12 @@ static void jammo_editing_track_get_property(GObject* object, guint prop_id, GVa
         }
 }
 static void jammo_editing_track_finalize(GObject* object) {
+	JammoEditingTrack* editing_track;
+
+	editing_track = JAMMO_EDITING_TRACK(object);
+
+	g_free(editing_track->priv->pitch);
+
 	G_OBJECT_CLASS(jammo_editing_track_parent_class)->finalize(object);
 }
 
@@ -308,6 +323,7 @@ static void jammo_editing_track_dispose(GObject* object) {
 	JammoEditingTrack* editing_track;
 
 	editing_track = JAMMO_EDITING_TRACK(object);
+
 	g_list_foreach(editing_track->priv->samples, (GFunc)_jammo_sample_set_editing_track, NULL);
 	g_list_foreach(editing_track->priv->samples, (GFunc)g_object_unref, NULL);
 	g_list_free(editing_track->priv->samples);
