@@ -260,7 +260,7 @@ static gboolean jammo_playing_track_setup_element(JammoTrack* track, GstBin* bin
 	
 	playing_track = JAMMO_PLAYING_TRACK(track);
 
-	g_return_if_fail(playing_track->priv->bin == NULL);
+	g_return_val_if_fail(playing_track->priv->bin == NULL, FALSE);
 	
 	playing_track->priv->bin = bin;
 	playing_track->priv->sink = sink_element;
