@@ -26,7 +26,7 @@
 static JammoInstrumentTrack *instrument_track;
 
 static void
-instrument_click (ClutterActor * actor, ClutterEvent * event, gpointer data)
+instrument_press (ClutterActor * actor, ClutterEvent * event, gpointer data)
 {
   int octave = GPOINTER_TO_INT (data) / 100;
   int note = GPOINTER_TO_INT (data) - 100 * octave;
@@ -37,6 +37,18 @@ instrument_click (ClutterActor * actor, ClutterEvent * event, gpointer data)
   jammo_instrument_track_note_on_realtime (instrument_track, notename);
 }
 
+static void
+instrument_release (ClutterActor * actor, ClutterEvent * event, gpointer data)
+{
+  int octave = GPOINTER_TO_INT (data) / 100;
+  int note = GPOINTER_TO_INT (data) - 100 * octave;
+  char notename = note+octave*12;
+
+	printf("octave: %d, note:%d, notename: %d\n", octave, note, notename);
+
+  jammo_instrument_track_note_off_realtime (instrument_track, notename);
+}
+
 
 static ClutterActor *
 make_instrument_grid (int type)
@@ -91,7 +103,10 @@ make_instrument_grid (int type)
 	  clutter_container_add_actor (CLUTTER_CONTAINER (container), box);
 	  clutter_actor_set_reactive (box, TRUE);
 	  g_signal_connect (box, "button-press-event",
-			    G_CALLBACK (instrument_click),
+			    G_CALLBACK (instrument_press),
+			    GINT_TO_POINTER (j * 100 + i));
+	  g_signal_connect (box, "button-release-event",
+			    G_CALLBACK (instrument_release),
 			    GINT_TO_POINTER (j * 100 + i));
 	}
     }
@@ -114,11 +129,11 @@ main (int argc, char **argv)
   clutter_actor_set_size (stage, 800.0, 480.0);
 
 
-  int type = 0;			//0= flute, 1=drumkit ,2=ud
+  int type = 3;			//0= flute, 1=drumkit ,2=ud, 3=piano
   sequencer = jammo_sequencer_new ();
   instrument_track = jammo_instrument_track_new (type);
-  jammo_instrument_track_set_realtime (instrument_track, TRUE);
   jammo_sequencer_add_track (sequencer, JAMMO_TRACK (instrument_track));
+  jammo_instrument_track_set_realtime (instrument_track, TRUE);
 
 
   clutter_container_add (CLUTTER_CONTAINER (stage),
